#!/bin/bash -ex
# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

# This scipt change the soft/hard limit of os and 
# add delete of rabbitmq user, set_permissions for rabbitmq user.

enable_plugin() {
    local plugin_name=$1
    if [ "X$plugin_name" = "X" ]; then
        return
    fi
    echo "Enabling plugin [$plugin_name]"
    $cli_rabbitmq_plugins enable $plugin_name
}

get_random_password() {
    # Generate 30 character random password.
    local random_password=$(date +%s | sha256sum | head -c 30; echo)
    echo "$random_password"
}

is_mq_user_present() {
    local uname=$1
    $rabbitmq_ctl --no-table-headers --quiet list_users | awk '{print $1}' | grep -q -e "^${uname}$"
    if [ $? -ne 0 ]; then
       # 1 means False
       return 1
    fi
    # 0 means True
    return 0
}

init() {
    cyops_guest="guest"
    cyops_user="cyops"
    cyops_role="administrator"
    cyops_vhost="intra-cyops"
    rabbitmq_config_file="rabbitmq-defaults"
    cyops_passwd_file="rabbitmq_users.conf"
    cyops_passwd_file_path="/opt/cyops/configs/rabbitmq"
    rabbitmq_ctl="/sbin/rabbitmqctl"
    cli_rabbitmq_plugins="/sbin/rabbitmq-plugins"
    rabbitmq_version=$($rabbitmq_ctl --version)
    cyops_rabbitmq_config="/usr/lib/rabbitmq/lib/rabbitmq_server-${rabbitmq_version}/sbin"
    f_sysctl="/etc/sysctl.conf"
    f_rb_service="/usr/lib/systemd/system/rabbitmq-server.service"
    f_limit="/etc/security/limits.conf"

    rb_user=$($rabbitmq_ctl --no-table-headers --quiet list_users | awk '{print $1}' | grep -e "^${cyops_user}$")
    rb_vhost=$($rabbitmq_ctl --no-table-headers --quiet list_vhosts | grep -e "^${cyops_vhost}$")

    f_release_file="/etc/cyops-release"
    s_product_type=$(cat $f_release_file | awk -F= '/^product_type=/ {print $2}')

}

set_os_limit() {
   # Change the OS and rabbitmq soft/hard limit
   os_limit=$(ulimit -a | grep -A 1 "max memory size" | grep "open files" | awk '{print $4}')
   # This is for fresh and update the CyOps.
   if [  $os_limit -le 65535 ]; then

      i_file_max=`grep fs.file-max $f_sysctl | awk '{print $3}'`
      if [ "$i_file_max" != "100000" ];then
         sed -i '$ i\fs.file-max = 100000' $f_sysctl
      fi

      grep -qw "LimitNOFILE=65535" $f_rb_service
      if [ $? != 0 ];then
         sed -i '/LimitNOFILE/a LimitNOFILE=65535' $f_rb_service
      fi

      i_soft_nproc=`grep "soft nproc 65535" $f_limit | awk '{print $4}'`
      i_hard_nproc=`grep "hard nproc 65535" $f_limit | awk '{print $4}'`
      i_soft_nofile=`grep "soft nofile 65535" $f_limit | awk '{print $4}'`
      i_hard_nofile=`grep "hard nofile 65535" $f_limit | awk '{print $4}'`
      if [ "$i_soft_nproc" != "65535" ] && [ "$i_hard_nproc" != "65535" ] && [ "$i_soft_nofile" != "65535" ] && [ "$i_hard_nofile" != "65535" ];then
         sed -i -e '$a\
         * soft nproc 65535\
         * hard nproc 65535\
         * soft nofile 65535\
         * hard nofile 65535' $f_limit
         sudo sysctl -p
         systemctl daemon-reload
      fi
   fi
}

delete_user_guest() {
    $rabbitmq_ctl delete_user $cyops_guest
}

create_ha_policy() {
    local vhost=$1
    if [ -z "$vhost" ]; then
        echo "Error: Virtual host name is required."
        return 1
    fi
    echo "Creating HA policy for vhost $vhost"
    csadm mq create-policy --name "${vhost}-ha-policy" --vhost "$vhost" --pattern '.*' --apply-to all --definition ha-mode=all ha-sync-mode=automatic >/dev/null 2>&1
}


create_db() {
    echo "User process started."
    if [ -f $cyops_passwd_file_path/$cyops_passwd_file ]; then
        cyops_passwd=$(grep "cyops.rabbitmq.password" $cyops_passwd_file_path/$cyops_passwd_file | cut -d"=" -f2)
        $rabbitmq_ctl add_user cyops $cyops_passwd
        $rabbitmq_ctl set_user_tags $cyops_user $cyops_role
        $rabbitmq_ctl add_vhost $cyops_vhost
        $rabbitmq_ctl set_permissions -p $cyops_vhost $cyops_user ".*" ".*" ".*"
        create_ha_policy "$cyops_vhost"
        sed -i "s/^\s*CONFIG_FILE\s*=\(.*\)\/etc\/rabbitmq.*$/CONFIG_FILE=\1\/etc\/rabbitmq\/rabbitmq.conf/g" $cyops_rabbitmq_config/$rabbitmq_config_file
        chown -R root:rabbitmq /opt/cyops/configs/rabbitmq/
    fi
}

create_user_wf() {
    local password_type="random"
    if [ "X$1" != "X" ]; then
        password_type=$1
    fi
    local current_node_db_config_yml="/opt/cyops/configs/database/db_config.yml"
    local default_db_config_yml="/opt/cyops/configs/database/db_config.yml.default"
    local conf_util="/opt/cyops/scripts/confUtil.py"



    local wf_uname=$(python3 $conf_util -g "mq_user" -f $current_node_db_config_yml)
    local wf_vhost=$(python3 $conf_util -g "mq_vhost" -f $current_node_db_config_yml)
    local wf_user_passwd=$(get_random_password)
    if [ "X$password_type" = "Xmap-to-device-uuid" ]; then
        # If this flag is passed, the password is set to be same as the device uuid of the fsr instance
        wf_user_passwd=$(csadm license --get-device-uuid)
    fi

    if is_mq_user_present "$wf_uname"; then
        # User already present, change the password
        $rabbitmq_ctl change_password "$wf_uname" "$wf_user_passwd"
    else
        # Create new user
        $rabbitmq_ctl add_user "$wf_uname" "$wf_user_passwd"
    fi
    $rabbitmq_ctl add_vhost "$wf_vhost"
    $rabbitmq_ctl set_permissions -p "$wf_vhost" "$wf_uname" ".*" ".*" ".*"
    create_ha_policy "$wf_vhost"
    chown -R root:rabbitmq /opt/cyops/configs/rabbitmq/

    local encrypted_wf_user_passwd=$(python3 /opt/cyops/scripts/manage_passwords.py --encrypt $wf_user_passwd)
    python3 $conf_util \
             -f $current_node_db_config_yml \
             -k "mq_password" -v $encrypted_wf_user_passwd
    if [ -f "$default_db_config_yml" ]; then
        #
        # This is required as when the node is part of the cluster, the MQ details in db_config.yml
        # file refers to primary node. The current node details are in db_config.yml.default file.
        # 'csadm mq db flush'. MQ flush db calls this script in order to create the users and details again.
        # New wf_user_passwd must be updated in db_config.yml.default
        python3 $conf_util \
             -f $default_db_config_yml \
             -k "mq_password" -v $encrypted_wf_user_passwd
    fi
}

handle_installation_fresh() {
    if [[ "X$rb_user" = "X$cyops_user" &&  "X$rb_vhost" = "X$cyops_vhost" ]]; then
        # This means both user cyops and vhost intra-cyops present, hence it indicates that
        # this script is run before. In order word, this is not the fresh installtion, hence return.
        return
    fi

    set_os_limit
    delete_user_guest
    enable_plugin "rabbitmq_auth_mechanism_ssl"

    if [ $s_product_type = "secure-message-exchange" ]; then
        return
    fi

    create_db
    create_user_wf
}

##################
# Main/main/MAIN starts here
##################

installation_type="fresh"
upgrade_type=""

# $1 is installation type which can be fresh/upgrade
if [ "X$1" != "X" ]; then
    installation_type=$1
fi

# $2 is upgrade_type which can be cyops/mq which will be honored only in case of upgrade
if [ "X$2" != "X" ]; then
    upgrade_type=$2
fi

init

if [ "X$installation_type" = "Xfresh" ]; then
    handle_installation_fresh
    exit 0
fi

if [ $s_product_type = "secure-message-exchange" ]; then
    # Below section is not applicable for SME
    exit 0
fi

if [ "X$installation_type" = "Xupgrade" ]; then
    case $upgrade_type in
        cyops)
            create_db
            ;;
        mq)
            create_user_wf "map-to-device-uuid"
            ;;
        all)
            create_db
            create_user_wf "map-to-device-uuid"
    esac
fi

exit 0
