# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end
source_csadm() {
    fsr_csadm_utils="/opt/cyops/adm/utils"
    source $fsr_csadm_utils
}

init() {
    source_csadm
}

# This can be removed once base version becomes 7.0.2
enable_mtls_for_forticloud() {
    if ! is_forticloud_instance; then
        # Not the FortiCloud instance, return
        return
    fi
    csadm mq mtls enable --skip-service-restart
}

# This can be removed once base version becomes 7.0.2
add_mtls_config() {
    local file_mq_config=$1
    if grep -q "EXTERNAL" $file_mq_config; then
        return
    fi

    local new_config="auth_mechanisms.1 = EXTERNAL
auth_mechanisms.2 = PLAIN
auth_mechanisms.3 = AMQPLAIN
ssl_cert_login_from = common_name
ssl_options.depth = 2"

    echo "" >> $file_mq_config
    echo "$new_config" >> $file_mq_config
}

validate_input() {
    local context=$1
    local version=$2
    if [ "X$context" = "X" ] || [ "X$version" = "X" ]; then
        echo "$0 upgrade <version-here>"
        exit 1
    fi
}

handle_upgrade() {
    local new_version=$1
    local fsr_default_ca_attr_config="/opt/cyops/configs/rabbitmq/ssl/cyopsca/index.txt.attr"
    # This can be removed once base version becomes 7.0.2
    append_or_replace "$fsr_default_ca_attr_config" "unique_subject" "no"

    # This can be removed once base version becomes 7.0.2
    rm -rf /opt/cyops-rabbitmq/configs/ssl/work/

    # This can be removed once base version becomes 7.0.2
    rm -f /etc/rabbitmq/openssl.cnf

    # This can be removed once base version becomes 7.0.2
    add_mtls_config "/opt/cyops-rabbitmq/configs/ssl/rabbitmq-embedded-sme.conf"

    # This can be removed once base version becomes 7.0.2
    add_mtls_config "/opt/cyops-rabbitmq/configs/ssl/rabbitmq-external-sme.conf" 

    # This can be removed once base version becomes 7.0.2
    enable_mtls_for_forticloud

    # This can be removed once base version becomes 7.0.2 
    /sbin/rabbitmq-plugins enable rabbitmq_auth_mechanism_ssl
  
    # This can be removed once base version becomes 7.0.2 
    local file_old_rbmq_db="/opt/cyops-rabbitmq/configs/rabbitmq/rabbitmq_db.conf"
    if [ -f "$file_old_rbmq_db" ]; then
        mv $file_old_rbmq_db /opt/cyops/configs/rabbitmq/
    fi
}

handle_default_steps_for_context_upgrade() {
    systemctl restart rabbitmq-server
}

############
# Main/main/MAIN starts here
#########
context=$1
version=$2
# This script usually runs from the specfile. Hence add -x here for debug.
set -x
init
validate_input "$context" "$version"
handle_upgrade "$version"
handle_default_steps_for_context_upgrade
exit 0
